@@ -49,7 +49,7 @@
 #' @param ... Additional arguments passed to `install.packages()`
 #' @export
 #' @importFrom utils install.packages
-#' @seealso [arrow_available()] to see if the package was configured with
+#' @seealso [arrow_info()] to see if the package was configured with
 #' necessary C++ dependencies. `vignette("install", package = "arrow")` for
 #' more ways to tune installation on Linux.
 install_arrow <- function(nightly = FALSE,
@@ -113,7 +113,7 @@ arrow_repos <- function(repos = getOption("repos"), nightly = FALSE) {
     # Set the default/CDN
     repos <- "https://cloud.r-project.org/"
   }
-  dev_repo <- getOption("arrow.dev_repo", "https://arrow-r-nightly.s3.amazonaws.com")
+  dev_repo <- getOption("arrow.dev_repo", "https://nightlies.apache.org/arrow/r")
   # Remove it if it's there (so nightly=FALSE won't accidentally pull from it)
   repos <- setdiff(repos, dev_repo)
   if (nightly) {
