@@ -27,6 +27,11 @@ std::shared_ptr<arrow::Schema> RecordBatchReader__schema(
   return reader->schema();
 }
 
+// [[arrow::export]]
+void RecordBatchReader__Close(const std::shared_ptr<arrow::RecordBatchReader>& reader) {
+  return arrow::StopIfNotOk(reader->Close());
+}
+
 // [[arrow::export]]
 std::shared_ptr<arrow::RecordBatch> RecordBatchReader__ReadNext(
     const std::shared_ptr<arrow::RecordBatchReader>& reader) {
@@ -65,7 +70,7 @@ class RFunctionRecordBatchReader : public arrow::RecordBatchReader {
 
   arrow::Status ReadNext(std::shared_ptr<arrow::RecordBatch>* batch_out) {
     auto batch = SafeCallIntoR<std::shared_ptr<arrow::RecordBatch>>([&]() {
-      cpp11::sexp result_sexp = fun_();
+      cpp11::sexp result_sexp = cpp11::function(fun_)();
       if (result_sexp == R_NilValue) {
         return std::shared_ptr<arrow::RecordBatch>(nullptr);
       } else if (!Rf_inherits(result_sexp, "RecordBatch")) {
@@ -89,7 +94,7 @@ class RFunctionRecordBatchReader : public arrow::RecordBatchReader {
   }
 
  private:
-  cpp11::function fun_;
+  cpp11::sexp fun_;
   std::shared_ptr<arrow::Schema> schema_;
 };
 
@@ -111,19 +116,77 @@ std::shared_ptr<arrow::Table> Table__from_RecordBatchReader(
   return ValueOrStop(reader->ToTable());
 }
 
+// Because the head() operation can leave a RecordBatchReader whose contents
+// will never be drained, we implement a wrapper class here that takes care
+// to (1) return only the requested number of rows (or fewer) and (2) Close
+// and release the underlying reader as soon as possible. This is mostly
+// useful for the ExecPlanReader, whose Close() method also requests
+// that the ExecPlan stop producing, but may also be useful for readers
+// that point to an open file and whose Close() or delete method releases
+// the file.
+class RecordBatchReaderHead : public arrow::RecordBatchReader {
+ public:
+  RecordBatchReaderHead(std::shared_ptr<arrow::RecordBatchReader> reader,
+                        int64_t num_rows)
+      : schema_(reader->schema()), reader_(reader), num_rows_(num_rows) {}
+
+  std::shared_ptr<arrow::Schema> schema() const override { return schema_; }
+
+  arrow::Status ReadNext(std::shared_ptr<arrow::RecordBatch>* batch_out) override {
+    if (!reader_) {
+      // Close() has been called
+      batch_out = nullptr;
+      return arrow::Status::OK();
+    }
+
+    ARROW_RETURN_NOT_OK(reader_->ReadNext(batch_out));
+    if (batch_out->get()) {
+      num_rows_ -= batch_out->get()->num_rows();
+      if (num_rows_ < 0) {
+        auto smaller_batch =
+            batch_out->get()->Slice(0, batch_out->get()->num_rows() + num_rows_);
+        *batch_out = smaller_batch;
+      }
+
+      if (num_rows_ <= 0) {
+        // We've run out of num_rows before batches
+        ARROW_RETURN_NOT_OK(Close());
+      }
+    } else {
+      // We've run out of batches before num_rows
+      ARROW_RETURN_NOT_OK(Close());
+    }
+
+    return arrow::Status::OK();
+  }
+
+  arrow::Status Close() override {
+    if (reader_) {
+      arrow::Status result = reader_->Close();
+      reader_.reset();
+      return result;
+    } else {
+      return arrow::Status::OK();
+    }
+  }
+
+ private:
+  std::shared_ptr<arrow::Schema> schema_;
+  std::shared_ptr<arrow::RecordBatchReader> reader_;
+  int64_t num_rows_;
+};
+
 // [[arrow::export]]
 std::shared_ptr<arrow::RecordBatchReader> RecordBatchReader__Head(
     const std::shared_ptr<arrow::RecordBatchReader>& reader, int64_t num_rows) {
-  std::vector<std::shared_ptr<arrow::RecordBatch>> batches;
-  std::shared_ptr<arrow::RecordBatch> this_batch;
-  while (num_rows > 0) {
-    this_batch = ValueOrStop(reader->Next());
-    if (this_batch == nullptr) break;
-    batches.push_back(this_batch->Slice(0, num_rows));
-    num_rows -= this_batch->num_rows();
+  if (num_rows <= 0) {
+    // If we are never going to pull any batches from this reader, close it
+    // immediately.
+    StopIfNotOk(reader->Close());
+    return ValueOrStop(arrow::RecordBatchReader::Make({}, reader->schema()));
+  } else {
+    return std::make_shared<RecordBatchReaderHead>(reader, num_rows);
   }
-  return ValueOrStop(
-      arrow::RecordBatchReader::Make(std::move(batches), reader->schema()));
 }
 
 // -------- RecordBatchStreamReader
