@@ -52,7 +52,11 @@ test_that("Table metadata", {
 
 test_that("Table R metadata", {
   tab <- Table$create(example_with_metadata)
-  expect_output(print(tab$metadata), "arrow_r_metadata")
+  expect_output(
+    print(tab$metadata),
+    "$r$columns$c$columns$c1$attributes$extra_attr",
+    fixed = TRUE
+  )
   expect_identical(as.data.frame(tab), example_with_metadata)
 })
 
@@ -96,6 +100,7 @@ test_that("Garbage R metadata doesn't break things", {
   )
   # serialize data like .serialize_arrow_r_metadata does, but don't call that
   # directly since it checks to ensure that the data is a list
+  tab <- Table$create(example_data[1:6])
   tab$metadata$r <- rawToChar(serialize("garbage", NULL, ascii = TRUE))
   expect_warning(
     expect_identical(as.data.frame(tab), example_data[1:6]),
@@ -249,8 +254,6 @@ test_that("Row-level metadata (does not) roundtrip in datasets", {
   skip_if_not_available("dataset")
   skip_if_not_available("parquet")
 
-  library(dplyr, warn.conflicts = FALSE)
-
   df <- tibble::tibble(
     metadata = list(
       structure(1, my_value_as_attr = 1),
@@ -264,39 +267,36 @@ test_that("Row-level metadata (does not) roundtrip in datasets", {
 
   dst_dir <- make_temp_dir()
 
-  withr::with_options(
-    list("arrow.preserve_row_level_metadata" = TRUE),
-    {
-      expect_warning(
-        write_dataset(df, dst_dir, partitioning = "part"),
-        "Row-level metadata is not compatible with datasets and will be discarded"
-      )
+  withr::local_options("arrow.preserve_row_level_metadata" = TRUE)
 
-      # Reset directory as previous write will have created some files and the default
-      # behavior is to error on existing
-      dst_dir <- make_temp_dir()
-      # but we need to write a dataset with row-level metadata to make sure when
-      # reading ones that have been written with them we warn appropriately
-      fake_func_name <- write_dataset
-      fake_func_name(df, dst_dir, partitioning = "part")
+  expect_warning(
+    write_dataset(df, dst_dir, partitioning = "part"),
+    "Row-level metadata is not compatible with datasets and will be discarded"
+  )
 
-      ds <- open_dataset(dst_dir)
-      expect_warning(
-        df_from_ds <- collect(ds),
-        "Row-level metadata is not compatible with this operation and has been ignored"
-      )
-      expect_equal(
-        arrange(df_from_ds, int),
-        arrange(df, int),
-        ignore_attr = TRUE
-      )
+  # Reset directory as previous write will have created some files and the default
+  # behavior is to error on existing
+  dst_dir <- make_temp_dir()
+  # but we need to write a dataset with row-level metadata to make sure when
+  # reading ones that have been written with them we warn appropriately
+  fake_func_name <- write_dataset
+  fake_func_name(df, dst_dir, partitioning = "part")
 
-      # however there is *no* warning if we don't select the metadata column
-      expect_warning(
-        df_from_ds <- ds %>% select(int) %>% collect(),
-        NA
-      )
-    }
+  ds <- open_dataset(dst_dir)
+  expect_warning(
+    df_from_ds <- collect(ds),
+    "Row-level metadata is not compatible with this operation and has been ignored"
+  )
+  expect_equal(
+    dplyr::arrange(df_from_ds, int),
+    dplyr::arrange(df, int),
+    ignore_attr = TRUE
+  )
+
+  # however there is *no* warning if we don't select the metadata column
+  expect_warning(
+    df_from_ds <- ds %>% dplyr::select(int) %>% dplyr::collect(),
+    NA
   )
 })
 
@@ -377,7 +377,7 @@ test_that("grouped_df metadata is recorded (efficiently)", {
   expect_s3_class(grouped, "grouped_df")
   grouped_tab <- Table$create(grouped)
   expect_r6_class(grouped_tab, "Table")
-  expect_equal(grouped_tab$r_metadata$attributes$.group_vars, "a")
+  expect_equal(grouped_tab$metadata$r$attributes$.group_vars, "a")
 })
 
 test_that("grouped_df non-arrow metadata is preserved", {
