@@ -17,6 +17,7 @@
 
 #include "arrow/compute/exec/exec_plan.h"
 
+#include <optional>
 #include <sstream>
 #include <unordered_map>
 #include <unordered_set>
@@ -24,15 +25,20 @@
 #include "arrow/compute/exec.h"
 #include "arrow/compute/exec/expression.h"
 #include "arrow/compute/exec/options.h"
+#include "arrow/compute/exec/task_util.h"
+#include "arrow/compute/exec/util.h"
 #include "arrow/compute/exec_internal.h"
 #include "arrow/compute/registry.h"
 #include "arrow/datum.h"
 #include "arrow/record_batch.h"
 #include "arrow/result.h"
+#include "arrow/table.h"
 #include "arrow/util/async_generator.h"
 #include "arrow/util/checked_cast.h"
+#include "arrow/util/key_value_metadata.h"
 #include "arrow/util/logging.h"
-#include "arrow/util/optional.h"
+#include "arrow/util/tracing_internal.h"
+#include "arrow/util/vector.h"
 
 namespace arrow {
 
@@ -43,7 +49,9 @@ namespace compute {
 namespace {
 
 struct ExecPlanImpl : public ExecPlan {
-  explicit ExecPlanImpl(ExecContext* exec_context) : ExecPlan(exec_context) {}
+  explicit ExecPlanImpl(ExecContext* exec_context,
+                        std::shared_ptr<const KeyValueMetadata> metadata = NULLPTR)
+      : ExecPlan(exec_context), metadata_(std::move(metadata)) {}
 
   ~ExecPlanImpl() override {
     if (started_ && !finished_.is_finished()) {
@@ -53,6 +61,9 @@ struct ExecPlanImpl : public ExecPlan {
     }
   }
 
+  size_t GetThreadIndex() { return thread_indexer_(); }
+  size_t max_concurrency() const { return thread_indexer_.Capacity(); }
+
   ExecNode* AddNode(std::unique_ptr<ExecNode> node) {
     if (node->label().empty()) {
       node->SetLabel(std::to_string(auto_label_counter_++));
@@ -67,6 +78,44 @@ struct ExecPlanImpl : public ExecPlan {
     return nodes_.back().get();
   }
 
+  Result<Future<>> BeginExternalTask() {
+    Future<> completion_future = Future<>::Make();
+    if (async_scheduler_->AddSimpleTask(
+            [completion_future] { return completion_future; })) {
+      return completion_future;
+    }
+    return Future<>{};
+  }
+
+  Status ScheduleTask(std::function<Status()> fn) {
+    auto executor = exec_context_->executor();
+    if (!executor) return fn();
+    // Adds a task which submits fn to the executor and tracks its progress.  If we're
+    // aborted then the task is ignored and fn is not executed.
+    async_scheduler_->AddSimpleTask(
+        [executor, fn]() { return executor->Submit(std::move(fn)); });
+    return Status::OK();
+  }
+
+  Status ScheduleTask(std::function<Status(size_t)> fn) {
+    std::function<Status()> indexed_fn = [this, fn]() {
+      size_t thread_index = GetThreadIndex();
+      return fn(thread_index);
+    };
+    return ScheduleTask(std::move(indexed_fn));
+  }
+
+  int RegisterTaskGroup(std::function<Status(size_t, int64_t)> task,
+                        std::function<Status(size_t)> on_finished) {
+    return task_scheduler_->RegisterTaskGroup(std::move(task), std::move(on_finished));
+  }
+
+  Status StartTaskGroup(int task_group_id, int64_t num_tasks) {
+    return task_scheduler_->StartTaskGroup(GetThreadIndex(), task_group_id, num_tasks);
+  }
+
+  util::AsyncTaskScheduler* async_scheduler() { return async_scheduler_.get(); }
+
   Status Validate() const {
     if (nodes_.empty()) {
       return Status::Invalid("ExecPlan has no node");
@@ -78,15 +127,50 @@ struct ExecPlanImpl : public ExecPlan {
   }
 
   Status StartProducing() {
+    START_COMPUTE_SPAN(span_, "ExecPlan", {{"plan", ToString()}});
+#ifdef ARROW_WITH_OPENTELEMETRY
+    if (HasMetadata()) {
+      auto pairs = metadata().get()->sorted_pairs();
+      opentelemetry::nostd::shared_ptr<opentelemetry::trace::Span> span =
+          ::arrow::internal::tracing::UnwrapSpan(span_.details.get());
+      std::for_each(std::begin(pairs), std::end(pairs),
+                    [span](std::pair<std::string, std::string> const& pair) {
+                      span->SetAttribute(pair.first, pair.second);
+                    });
+    }
+#endif
     if (started_) {
       return Status::Invalid("restarted ExecPlan");
     }
-    started_ = true;
-
-    // producers precede consumers
-    sorted_nodes_ = TopoSort();
 
     std::vector<Future<>> futures;
+    for (auto& n : nodes_) {
+      RETURN_NOT_OK(n->Init());
+      futures.push_back(n->finished());
+    }
+
+    AllFinished(futures).AddCallback([this](const Status& st) {
+      error_st_ = st;
+      EndTaskGroup();
+    });
+
+    task_scheduler_->RegisterEnd();
+    int num_threads = 1;
+    bool sync_execution = true;
+    if (auto executor = exec_context()->executor()) {
+      num_threads = executor->GetCapacity();
+      sync_execution = false;
+    }
+    RETURN_NOT_OK(task_scheduler_->StartScheduling(
+        0 /* thread_index */,
+        [this](std::function<Status(size_t)> fn) -> Status {
+          return this->ScheduleTask(std::move(fn));
+        },
+        /*concurrent_tasks=*/2 * num_threads, sync_execution));
+
+    started_ = true;
+    // producers precede consumers
+    sorted_nodes_ = TopoSort();
 
     Status st = Status::OK();
 
@@ -94,32 +178,51 @@ struct ExecPlanImpl : public ExecPlan {
     for (rev_it it(sorted_nodes_.end()), end(sorted_nodes_.begin()); it != end; ++it) {
       auto node = *it;
 
+      EVENT(span_, "StartProducing:" + node->label(),
+            {{"node.label", node->label()}, {"node.kind_name", node->kind_name()}});
       st = node->StartProducing();
+      EVENT(span_, "StartProducing:" + node->label(), {{"status", st.ToString()}});
       if (!st.ok()) {
         // Stop nodes that successfully started, in reverse order
         stopped_ = true;
         StopProducingImpl(it.base(), sorted_nodes_.end());
-        break;
+        for (NodeVector::iterator fw_it = sorted_nodes_.begin(); fw_it != it.base();
+             ++fw_it) {
+          Future<> fut = (*fw_it)->finished();
+          if (!fut.is_finished()) fut.MarkFinished();
+        }
+        return st;
       }
-
-      futures.push_back(node->finished());
     }
-
-    finished_ = AllFinished(futures);
     return st;
   }
 
+  void EndTaskGroup() {
+    bool expected = false;
+    if (group_ended_.compare_exchange_strong(expected, true)) {
+      async_scheduler_->End();
+      async_scheduler_->OnFinished().AddCallback([this](const Status& st) {
+        MARK_SPAN(span_, error_st_ & st);
+        END_SPAN(span_);
+        finished_.MarkFinished(error_st_ & st);
+      });
+    }
+  }
+
   void StopProducing() {
     DCHECK(started_) << "stopped an ExecPlan which never started";
+    EVENT(span_, "StopProducing");
     stopped_ = true;
-
-    StopProducingImpl(sorted_nodes_.begin(), sorted_nodes_.end());
+    task_scheduler_->Abort(
+        [this]() { StopProducingImpl(sorted_nodes_.begin(), sorted_nodes_.end()); });
   }
 
   template <typename It>
   void StopProducingImpl(It begin, It end) {
     for (auto it = begin; it != end; ++it) {
       auto node = *it;
+      EVENT(span_, "StopProducing:" + node->label(),
+            {{"node.label", node->label()}, {"node.kind_name", node->kind_name()}});
       node->StopProducing();
     }
   }
@@ -217,12 +320,21 @@ struct ExecPlanImpl : public ExecPlan {
     return ss.str();
   }
 
-  Future<> finished_ = Future<>::MakeFinished();
+  Status error_st_;
+  Future<> finished_ = Future<>::Make();
   bool started_ = false, stopped_ = false;
   std::vector<std::unique_ptr<ExecNode>> nodes_;
   NodeVector sources_, sinks_;
   NodeVector sorted_nodes_;
   uint32_t auto_label_counter_ = 0;
+  util::tracing::Span span_;
+  std::shared_ptr<const KeyValueMetadata> metadata_;
+
+  ThreadIndexer thread_indexer_;
+  std::atomic<bool> group_ended_{false};
+  std::unique_ptr<util::AsyncTaskScheduler> async_scheduler_ =
+      util::AsyncTaskScheduler::Make();
+  std::unique_ptr<TaskScheduler> task_scheduler_ = TaskScheduler::Make();
 };
 
 ExecPlanImpl* ToDerived(ExecPlan* ptr) { return checked_cast<ExecPlanImpl*>(ptr); }
@@ -231,18 +343,21 @@ const ExecPlanImpl* ToDerived(const ExecPlan* ptr) {
   return checked_cast<const ExecPlanImpl*>(ptr);
 }
 
-util::optional<int> GetNodeIndex(const std::vector<ExecNode*>& nodes,
-                                 const ExecNode* node) {
+std::optional<int> GetNodeIndex(const std::vector<ExecNode*>& nodes,
+                                const ExecNode* node) {
   for (int i = 0; i < static_cast<int>(nodes.size()); ++i) {
     if (nodes[i] == node) return i;
   }
-  return util::nullopt;
+  return std::nullopt;
 }
 
 }  // namespace
 
-Result<std::shared_ptr<ExecPlan>> ExecPlan::Make(ExecContext* ctx) {
-  return std::shared_ptr<ExecPlan>(new ExecPlanImpl{ctx});
+const uint32_t ExecPlan::kMaxBatchSize;
+
+Result<std::shared_ptr<ExecPlan>> ExecPlan::Make(
+    ExecContext* ctx, std::shared_ptr<const KeyValueMetadata> metadata) {
+  return std::shared_ptr<ExecPlan>(new ExecPlanImpl{ctx, metadata});
 }
 
 ExecNode* ExecPlan::AddNode(std::unique_ptr<ExecNode> node) {
@@ -255,6 +370,31 @@ const ExecPlan::NodeVector& ExecPlan::sources() const {
 
 const ExecPlan::NodeVector& ExecPlan::sinks() const { return ToDerived(this)->sinks_; }
 
+size_t ExecPlan::GetThreadIndex() { return ToDerived(this)->GetThreadIndex(); }
+size_t ExecPlan::max_concurrency() const { return ToDerived(this)->max_concurrency(); }
+
+Result<Future<>> ExecPlan::BeginExternalTask() {
+  return ToDerived(this)->BeginExternalTask();
+}
+
+Status ExecPlan::ScheduleTask(std::function<Status()> fn) {
+  return ToDerived(this)->ScheduleTask(std::move(fn));
+}
+Status ExecPlan::ScheduleTask(std::function<Status(size_t)> fn) {
+  return ToDerived(this)->ScheduleTask(std::move(fn));
+}
+int ExecPlan::RegisterTaskGroup(std::function<Status(size_t, int64_t)> task,
+                                std::function<Status(size_t)> on_finished) {
+  return ToDerived(this)->RegisterTaskGroup(std::move(task), std::move(on_finished));
+}
+Status ExecPlan::StartTaskGroup(int task_group_id, int64_t num_tasks) {
+  return ToDerived(this)->StartTaskGroup(task_group_id, num_tasks);
+}
+
+util::AsyncTaskScheduler* ExecPlan::async_scheduler() {
+  return ToDerived(this)->async_scheduler();
+}
+
 Status ExecPlan::Validate() { return ToDerived(this)->Validate(); }
 
 Status ExecPlan::StartProducing() { return ToDerived(this)->StartProducing(); }
@@ -263,6 +403,12 @@ void ExecPlan::StopProducing() { ToDerived(this)->StopProducing(); }
 
 Future<> ExecPlan::finished() { return ToDerived(this)->finished_; }
 
+bool ExecPlan::HasMetadata() const { return !!(ToDerived(this)->metadata_); }
+
+std::shared_ptr<const KeyValueMetadata> ExecPlan::metadata() const {
+  return ToDerived(this)->metadata_;
+}
+
 std::string ExecPlan::ToString() const { return ToDerived(this)->ToString(); }
 
 ExecNode::ExecNode(ExecPlan* plan, NodeVector inputs,
@@ -278,6 +424,8 @@ ExecNode::ExecNode(ExecPlan* plan, NodeVector inputs,
   }
 }
 
+Status ExecNode::Init() { return Status::OK(); }
+
 Status ExecNode::Validate() const {
   if (inputs_.size() != input_labels_.size()) {
     return Status::Invalid("Invalid number of inputs for '", label(), "' (expected ",
@@ -330,114 +478,9 @@ bool ExecNode::ErrorIfNotOk(Status status) {
   return true;
 }
 
-MapNode::MapNode(ExecPlan* plan, std::vector<ExecNode*> inputs,
-                 std::shared_ptr<Schema> output_schema, bool async_mode)
-    : ExecNode(plan, std::move(inputs), /*input_labels=*/{"target"},
-               std::move(output_schema),
-               /*num_outputs=*/1) {
-  if (async_mode) {
-    executor_ = plan_->exec_context()->executor();
-  } else {
-    executor_ = nullptr;
-  }
-}
-
-void MapNode::ErrorReceived(ExecNode* input, Status error) {
-  DCHECK_EQ(input, inputs_[0]);
-  outputs_[0]->ErrorReceived(this, std::move(error));
-}
-
-void MapNode::InputFinished(ExecNode* input, int total_batches) {
-  DCHECK_EQ(input, inputs_[0]);
-  outputs_[0]->InputFinished(this, total_batches);
-  if (input_counter_.SetTotal(total_batches)) {
-    this->Finish();
-  }
-}
-
-Status MapNode::StartProducing() { return Status::OK(); }
-
-void MapNode::PauseProducing(ExecNode* output) {}
-
-void MapNode::ResumeProducing(ExecNode* output) {}
-
-void MapNode::StopProducing(ExecNode* output) {
-  DCHECK_EQ(output, outputs_[0]);
-  StopProducing();
-}
-
-void MapNode::StopProducing() {
-  if (executor_) {
-    this->stop_source_.RequestStop();
-  }
-  if (input_counter_.Cancel()) {
-    this->Finish();
-  }
-  inputs_[0]->StopProducing(this);
-}
-
-Future<> MapNode::finished() { return finished_; }
-
-void MapNode::SubmitTask(std::function<Result<ExecBatch>(ExecBatch)> map_fn,
-                         ExecBatch batch) {
-  Status status;
-  // This will be true if the node is stopped early due to an error or manual
-  // cancellation
-  if (input_counter_.Completed()) {
-    return;
-  }
-  auto task = [this, map_fn, batch]() {
-    auto guarantee = batch.guarantee;
-    auto output_batch = map_fn(std::move(batch));
-    if (ErrorIfNotOk(output_batch.status())) {
-      return output_batch.status();
-    }
-    output_batch->guarantee = guarantee;
-    outputs_[0]->InputReceived(this, output_batch.MoveValueUnsafe());
-    return Status::OK();
-  };
-
-  if (executor_) {
-    status = task_group_.AddTask([this, task]() -> Result<Future<>> {
-      return this->executor_->Submit(this->stop_source_.token(), [this, task]() {
-        auto status = task();
-        if (this->input_counter_.Increment()) {
-          this->Finish(status);
-        }
-        return status;
-      });
-    });
-  } else {
-    status = task();
-    if (input_counter_.Increment()) {
-      this->Finish(status);
-    }
-  }
-  // If we get a cancelled status from AddTask it means this node was stopped
-  // or errored out already so we can just drop the task.
-  if (!status.ok() && !status.IsCancelled()) {
-    if (input_counter_.Cancel()) {
-      this->Finish(status);
-    }
-    inputs_[0]->StopProducing(this);
-    return;
-  }
-}
-
-void MapNode::Finish(Status finish_st /*= Status::OK()*/) {
-  if (executor_) {
-    task_group_.End().AddCallback([this, finish_st](const Status& st) {
-      Status final_status = finish_st & st;
-      this->finished_.MarkFinished(final_status);
-    });
-  } else {
-    this->finished_.MarkFinished(finish_st);
-  }
-}
-
 std::shared_ptr<RecordBatchReader> MakeGeneratorReader(
-    std::shared_ptr<Schema> schema,
-    std::function<Future<util::optional<ExecBatch>>()> gen, MemoryPool* pool) {
+    std::shared_ptr<Schema> schema, std::function<Future<std::optional<ExecBatch>>()> gen,
+    MemoryPool* pool) {
   struct Impl : RecordBatchReader {
     std::shared_ptr<Schema> schema() const override { return schema_; }
 
@@ -451,9 +494,19 @@ std::shared_ptr<RecordBatchReader> MakeGeneratorReader(
       return Status::OK();
     }
 
+    Status Close() override {
+      // reading from generator until end is reached.
+      std::shared_ptr<RecordBatch> batch;
+      RETURN_NOT_OK(ReadNext(&batch));
+      while (batch != NULLPTR) {
+        RETURN_NOT_OK(ReadNext(&batch));
+      }
+      return Status::OK();
+    }
+
     MemoryPool* pool_;
     std::shared_ptr<Schema> schema_;
-    Iterator<util::optional<ExecBatch>> iterator_;
+    Iterator<std::optional<ExecBatch>> iterator_;
   };
 
   auto out = std::make_shared<Impl>();
@@ -469,12 +522,12 @@ Result<ExecNode*> Declaration::AddToPlan(ExecPlan* plan,
 
   size_t i = 0;
   for (const Input& input : this->inputs) {
-    if (auto node = util::get_if<ExecNode*>(&input)) {
+    if (auto node = std::get_if<ExecNode*>(&input)) {
       inputs[i++] = *node;
       continue;
     }
     ARROW_ASSIGN_OR_RAISE(inputs[i++],
-                          util::get<Declaration>(input).AddToPlan(plan, registry));
+                          std::get<Declaration>(input).AddToPlan(plan, registry));
   }
 
   ARROW_ASSIGN_OR_RAISE(
@@ -495,11 +548,70 @@ Declaration Declaration::Sequence(std::vector<Declaration> decls) {
     decls.pop_back();
 
     receiver->inputs.emplace_back(std::move(input));
-    receiver = &util::get<Declaration>(receiver->inputs.front());
+    receiver = &std::get<Declaration>(receiver->inputs.front());
   }
   return out;
 }
 
+bool Declaration::IsValid(ExecFactoryRegistry* registry) const {
+  return !this->factory_name.empty() && this->options != nullptr;
+}
+
+Future<std::shared_ptr<Table>> DeclarationToTableAsync(Declaration declaration,
+                                                       ExecContext* exec_context) {
+  std::shared_ptr<std::shared_ptr<Table>> output_table =
+      std::make_shared<std::shared_ptr<Table>>();
+  ARROW_ASSIGN_OR_RAISE(std::shared_ptr<ExecPlan> exec_plan,
+                        ExecPlan::Make(exec_context));
+  Declaration with_sink = Declaration::Sequence(
+      {declaration, {"table_sink", TableSinkNodeOptions(output_table.get())}});
+  ARROW_RETURN_NOT_OK(with_sink.AddToPlan(exec_plan.get()));
+  ARROW_RETURN_NOT_OK(exec_plan->StartProducing());
+  return exec_plan->finished().Then([exec_plan, output_table] { return *output_table; });
+}
+
+Result<std::shared_ptr<Table>> DeclarationToTable(Declaration declaration,
+                                                  ExecContext* exec_context) {
+  return DeclarationToTableAsync(std::move(declaration), exec_context).result();
+}
+
+Future<std::vector<std::shared_ptr<RecordBatch>>> DeclarationToBatchesAsync(
+    Declaration declaration, ExecContext* exec_context) {
+  return DeclarationToTableAsync(std::move(declaration), exec_context)
+      .Then([](const std::shared_ptr<Table>& table) {
+        return TableBatchReader(table).ToRecordBatches();
+      });
+}
+
+Result<std::vector<std::shared_ptr<RecordBatch>>> DeclarationToBatches(
+    Declaration declaration, ExecContext* exec_context) {
+  return DeclarationToBatchesAsync(std::move(declaration), exec_context).result();
+}
+
+Future<std::vector<ExecBatch>> DeclarationToExecBatchesAsync(Declaration declaration,
+                                                             ExecContext* exec_context) {
+  AsyncGenerator<std::optional<ExecBatch>> sink_gen;
+  ARROW_ASSIGN_OR_RAISE(std::shared_ptr<ExecPlan> exec_plan,
+                        ExecPlan::Make(exec_context));
+  Declaration with_sink =
+      Declaration::Sequence({declaration, {"sink", SinkNodeOptions(&sink_gen)}});
+  ARROW_RETURN_NOT_OK(with_sink.AddToPlan(exec_plan.get()));
+  ARROW_RETURN_NOT_OK(exec_plan->StartProducing());
+  auto collected_fut = CollectAsyncGenerator(sink_gen);
+  return AllFinished({exec_plan->finished(), Future<>(collected_fut)})
+      .Then([collected_fut, exec_plan]() -> Result<std::vector<ExecBatch>> {
+        ARROW_ASSIGN_OR_RAISE(auto collected, collected_fut.result());
+        return ::arrow::internal::MapVector(
+            [](std::optional<ExecBatch> batch) { return std::move(*batch); },
+            std::move(collected));
+      });
+}
+
+Result<std::vector<ExecBatch>> DeclarationToExecBatches(Declaration declaration,
+                                                        ExecContext* exec_context) {
+  return DeclarationToExecBatchesAsync(std::move(declaration), exec_context).result();
+}
+
 namespace internal {
 
 void RegisterSourceNode(ExecFactoryRegistry*);
@@ -509,6 +621,7 @@ void RegisterUnionNode(ExecFactoryRegistry*);
 void RegisterAggregateNode(ExecFactoryRegistry*);
 void RegisterSinkNode(ExecFactoryRegistry*);
 void RegisterHashJoinNode(ExecFactoryRegistry*);
+void RegisterAsofJoinNode(ExecFactoryRegistry*);
 
 }  // namespace internal
 
@@ -523,6 +636,7 @@ ExecFactoryRegistry* default_exec_factory_registry() {
       internal::RegisterAggregateNode(this);
       internal::RegisterSinkNode(this);
       internal::RegisterHashJoinNode(this);
+      internal::RegisterAsofJoinNode(this);
     }
 
     Result<Factory> GetFactory(const std::string& factory_name) override {
@@ -554,12 +668,12 @@ ExecFactoryRegistry* default_exec_factory_registry() {
   return &instance;
 }
 
-Result<std::function<Future<util::optional<ExecBatch>>()>> MakeReaderGenerator(
+Result<std::function<Future<std::optional<ExecBatch>>()>> MakeReaderGenerator(
     std::shared_ptr<RecordBatchReader> reader, ::arrow::internal::Executor* io_executor,
     int max_q, int q_restart) {
   auto batch_it = MakeMapIterator(
       [](std::shared_ptr<RecordBatch> batch) {
-        return util::make_optional(ExecBatch(*batch));
+        return std::make_optional(ExecBatch(*batch));
       },
       MakeIteratorFromReader(reader));
 
