@@ -19,12 +19,12 @@
 
 #include "arrow/api.h"
 #include "arrow/compute/exec/hash_join.h"
+#include "arrow/compute/exec/hash_join_node.h"
 #include "arrow/compute/exec/options.h"
 #include "arrow/compute/exec/test_util.h"
 #include "arrow/compute/exec/util.h"
 #include "arrow/compute/kernels/row_encoder.h"
 #include "arrow/testing/random.h"
-#include "arrow/util/make_unique.h"
 #include "arrow/util/thread_pool.h"
 
 #include <cstdint>
@@ -38,6 +38,9 @@ namespace compute {
 struct BenchmarkSettings {
   int num_threads = 1;
   JoinType join_type = JoinType::INNER;
+  // Change to 'true' to benchmark alternative, non-default and less optimized version of
+  // a hash join node implementation.
+  bool use_basic_implementation = false;
   int batch_size = 1024;
   int num_build_batches = 32;
   int num_probe_batches = 32 * 16;
@@ -53,10 +56,9 @@ struct BenchmarkSettings {
 class JoinBenchmark {
  public:
   explicit JoinBenchmark(BenchmarkSettings& settings) {
-    bool is_parallel = settings.num_threads != 1;
-
     SchemaBuilder l_schema_builder, r_schema_builder;
     std::vector<FieldRef> left_keys, right_keys;
+    std::vector<JoinKeyCmp> key_cmp;
     for (size_t i = 0; i < settings.key_types.size(); i++) {
       std::string l_name = "lk" + std::to_string(i);
       std::string r_name = "rk" + std::to_string(i);
@@ -77,6 +79,8 @@ class JoinBenchmark {
       build_metadata["null_probability"] = std::to_string(settings.null_percentage);
       build_metadata["min"] = std::to_string(min_build_value);
       build_metadata["max"] = std::to_string(max_build_value);
+      build_metadata["min_length"] = "2";
+      build_metadata["max_length"] = "20";
 
       std::unordered_map<std::string, std::string> probe_metadata;
       probe_metadata["null_probability"] = std::to_string(settings.null_percentage);
@@ -93,6 +97,7 @@ class JoinBenchmark {
 
       left_keys.push_back(FieldRef(l_name));
       right_keys.push_back(FieldRef(r_name));
+      key_cmp.push_back(JoinKeyCmp::EQ);
     }
 
     for (size_t i = 0; i < settings.build_payload_types.size(); i++) {
@@ -108,23 +113,32 @@ class JoinBenchmark {
     auto l_schema = *l_schema_builder.Finish();
     auto r_schema = *r_schema_builder.Finish();
 
-    l_batches_ =
+    BatchesWithSchema l_batches_with_schema =
         MakeRandomBatches(l_schema, settings.num_probe_batches, settings.batch_size);
-    r_batches_ =
+    BatchesWithSchema r_batches_with_schema =
         MakeRandomBatches(r_schema, settings.num_build_batches, settings.batch_size);
 
+    for (ExecBatch& batch : l_batches_with_schema.batches)
+      l_batches_.InsertBatch(std::move(batch));
+    for (ExecBatch& batch : r_batches_with_schema.batches)
+      r_batches_.InsertBatch(std::move(batch));
+
     stats_.num_probe_rows = settings.num_probe_batches * settings.batch_size;
 
-    ctx_ = arrow::internal::make_unique<ExecContext>(
-        default_memory_pool(),
-        is_parallel ? arrow::internal::GetCpuThreadPool() : nullptr);
+    ctx_ = std::make_unique<ExecContext>(default_memory_pool(),
+                                         arrow::internal::GetCpuThreadPool());
 
-    schema_mgr_ = arrow::internal::make_unique<HashJoinSchema>();
+    schema_mgr_ = std::make_unique<HashJoinSchema>();
     Expression filter = literal(true);
-    DCHECK_OK(schema_mgr_->Init(settings.join_type, *l_batches_.schema, left_keys,
-                                *r_batches_.schema, right_keys, filter, "l_", "r_"));
+    DCHECK_OK(schema_mgr_->Init(settings.join_type, *l_batches_with_schema.schema,
+                                left_keys, *r_batches_with_schema.schema, right_keys,
+                                filter, "l_", "r_"));
 
-    join_ = *HashJoinImpl::MakeBasic();
+    if (settings.use_basic_implementation) {
+      join_ = *HashJoinImpl::MakeBasic();
+    } else {
+      join_ = *HashJoinImpl::MakeSwiss();
+    }
 
     omp_set_num_threads(settings.num_threads);
     auto schedule_callback = [](std::function<Status(size_t)> func) -> Status {
@@ -133,38 +147,60 @@ class JoinBenchmark {
       return Status::OK();
     };
 
+    scheduler_ = TaskScheduler::Make();
+
+    auto register_task_group_callback = [&](std::function<Status(size_t, int64_t)> task,
+                                            std::function<Status(size_t)> cont) {
+      return scheduler_->RegisterTaskGroup(std::move(task), std::move(cont));
+    };
+
+    auto start_task_group_callback = [&](int task_group_id, int64_t num_tasks) {
+      return scheduler_->StartTaskGroup(omp_get_thread_num(), task_group_id, num_tasks);
+    };
+
     DCHECK_OK(join_->Init(
-        ctx_.get(), settings.join_type, !is_parallel, settings.num_threads,
-        schema_mgr_.get(), {JoinKeyCmp::EQ}, std::move(filter), [](ExecBatch) {},
-        [](int64_t x) {}, schedule_callback));
+        ctx_.get(), settings.join_type, settings.num_threads,
+        &(schema_mgr_->proj_maps[0]), &(schema_mgr_->proj_maps[1]), std::move(key_cmp),
+        std::move(filter), std::move(register_task_group_callback),
+        std::move(start_task_group_callback), [](int64_t, ExecBatch) {},
+        [](int64_t x) {}));
+
+    task_group_probe_ = scheduler_->RegisterTaskGroup(
+        [this](size_t thread_index, int64_t task_id) -> Status {
+          return join_->ProbeSingleBatch(thread_index, std::move(l_batches_[task_id]));
+        },
+        [this](size_t thread_index) -> Status {
+          return join_->ProbingFinished(thread_index);
+        });
+
+    scheduler_->RegisterEnd();
+
+    DCHECK_OK(scheduler_->StartScheduling(
+        0 /*thread index*/, std::move(schedule_callback),
+        static_cast<int>(2 * settings.num_threads) /*concurrent tasks*/,
+        settings.num_threads == 1));
   }
 
   void RunJoin() {
 #pragma omp parallel
     {
       int tid = omp_get_thread_num();
-#pragma omp for nowait
-      for (auto it = r_batches_.batches.begin(); it != r_batches_.batches.end(); ++it)
-        DCHECK_OK(join_->InputReceived(tid, /*side=*/1, *it));
-#pragma omp for nowait
-      for (auto it = l_batches_.batches.begin(); it != l_batches_.batches.end(); ++it)
-        DCHECK_OK(join_->InputReceived(tid, /*side=*/0, *it));
-
-#pragma omp barrier
-
-#pragma omp single nowait
-      { DCHECK_OK(join_->InputFinished(tid, /*side=*/1)); }
-
-#pragma omp single nowait
-      { DCHECK_OK(join_->InputFinished(tid, /*side=*/0)); }
+#pragma omp single
+      DCHECK_OK(
+          join_->BuildHashTable(tid, std::move(r_batches_), [this](size_t thread_index) {
+            return scheduler_->StartTaskGroup(thread_index, task_group_probe_,
+                                              l_batches_.batch_count());
+          }));
     }
   }
 
-  BatchesWithSchema l_batches_;
-  BatchesWithSchema r_batches_;
+  std::unique_ptr<TaskScheduler> scheduler_;
+  AccumulationQueue l_batches_;
+  AccumulationQueue r_batches_;
   std::unique_ptr<HashJoinSchema> schema_mgr_;
   std::unique_ptr<HashJoinImpl> join_;
   std::unique_ptr<ExecContext> ctx_;
+  int task_group_probe_;
 
   struct {
     uint64_t num_probe_rows;
@@ -173,11 +209,17 @@ class JoinBenchmark {
 
 static void HashJoinBasicBenchmarkImpl(benchmark::State& st,
                                        BenchmarkSettings& settings) {
-  JoinBenchmark bm(settings);
   uint64_t total_rows = 0;
   for (auto _ : st) {
-    bm.RunJoin();
-    total_rows += bm.stats_.num_probe_rows;
+    st.PauseTiming();
+    {
+      JoinBenchmark bm(settings);
+      st.ResumeTiming();
+      bm.RunJoin();
+      st.PauseTiming();
+      total_rows += bm.stats_.num_probe_rows;
+    }
+    st.ResumeTiming();
   }
   st.counters["rows/sec"] = benchmark::Counter(total_rows, benchmark::Counter::kIsRate);
 }
