@@ -33,7 +33,7 @@ constexpr int64_t kStringTransformError = -1;
 
 struct StringTransformBase {
   virtual ~StringTransformBase() = default;
-  virtual Status PreExec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
+  virtual Status PreExec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     return Status::OK();
   }
 
@@ -48,6 +48,12 @@ struct StringTransformBase {
   }
 };
 
+template <typename offset_type>
+static int64_t GetVarBinaryValuesLength(const ArraySpan& span) {
+  const offset_type* offsets = span.GetValues<offset_type>(1);
+  return span.length > 0 ? offsets[span.length] - offsets[0] : 0;
+}
+
 /// Kernel exec generator for unary string transforms. Types of template
 /// parameter StringTransform need to define a transform method with the
 /// following signature:
@@ -68,24 +74,17 @@ struct StringTransformExecBase {
   using ArrayType = typename TypeTraits<Type>::ArrayType;
 
   static Status Execute(KernelContext* ctx, StringTransform* transform,
-                        const ExecBatch& batch, Datum* out) {
-    if (batch[0].kind() == Datum::ARRAY) {
-      return ExecArray(ctx, transform, batch[0].array(), out);
-    }
-    DCHECK_EQ(batch[0].kind(), Datum::SCALAR);
-    return ExecScalar(ctx, transform, batch[0].scalar(), out);
-  }
+                        const ExecSpan& batch, ExecResult* out) {
+    const ArraySpan& input = batch[0].array;
+    auto offsets = input.GetValues<offset_type>(1);
+    const uint8_t* input_data = input.buffers[2].data;
 
-  static Status ExecArray(KernelContext* ctx, StringTransform* transform,
-                          const std::shared_ptr<ArrayData>& data, Datum* out) {
-    ArrayType input(data);
-    const int64_t input_ncodeunits = input.total_values_length();
-    const int64_t input_nstrings = input.length();
+    const int64_t input_ncodeunits = GetVarBinaryValuesLength<offset_type>(input);
     const int64_t max_output_ncodeunits =
-        transform->MaxCodeunits(input_nstrings, input_ncodeunits);
+        transform->MaxCodeunits(input.length, input_ncodeunits);
     RETURN_NOT_OK(CheckOutputCapacity(max_output_ncodeunits));
 
-    ArrayData* output = out->mutable_array();
+    ArrayData* output = out->array_data().get();
     ARROW_ASSIGN_OR_RAISE(auto values_buffer, ctx->Allocate(max_output_ncodeunits));
     output->buffers[2] = values_buffer;
 
@@ -94,10 +93,10 @@ struct StringTransformExecBase {
     uint8_t* output_str = output->buffers[2]->mutable_data();
     offset_type output_ncodeunits = 0;
     output_string_offsets[0] = output_ncodeunits;
-    for (int64_t i = 0; i < input_nstrings; i++) {
+    for (int64_t i = 0; i < input.length; i++) {
       if (!input.IsNull(i)) {
-        offset_type input_string_ncodeunits;
-        const uint8_t* input_string = input.GetValue(i, &input_string_ncodeunits);
+        const uint8_t* input_string = input_data + offsets[i];
+        offset_type input_string_ncodeunits = offsets[i + 1] - offsets[i];
         auto encoded_nbytes = static_cast<offset_type>(transform->Transform(
             input_string, input_string_ncodeunits, output_str + output_ncodeunits));
         if (encoded_nbytes < 0) {
@@ -113,29 +112,6 @@ struct StringTransformExecBase {
     return values_buffer->Resize(output_ncodeunits, /*shrink_to_fit=*/true);
   }
 
-  static Status ExecScalar(KernelContext* ctx, StringTransform* transform,
-                           const std::shared_ptr<Scalar>& scalar, Datum* out) {
-    const auto& input = checked_cast<const BaseBinaryScalar&>(*scalar);
-    if (!input.is_valid) {
-      return Status::OK();
-    }
-    const int64_t data_nbytes = static_cast<int64_t>(input.value->size());
-    const int64_t max_output_ncodeunits = transform->MaxCodeunits(1, data_nbytes);
-    RETURN_NOT_OK(CheckOutputCapacity(max_output_ncodeunits));
-
-    ARROW_ASSIGN_OR_RAISE(auto value_buffer, ctx->Allocate(max_output_ncodeunits));
-    auto* result = checked_cast<BaseBinaryScalar*>(out->scalar().get());
-    result->is_valid = true;
-    result->value = value_buffer;
-    auto encoded_nbytes = static_cast<offset_type>(transform->Transform(
-        input.value->data(), data_nbytes, value_buffer->mutable_data()));
-    if (encoded_nbytes < 0) {
-      return transform->InvalidInputSequence();
-    }
-    DCHECK_LE(encoded_nbytes, max_output_ncodeunits);
-    return value_buffer->Resize(encoded_nbytes, /*shrink_to_fit=*/true);
-  }
-
   static Status CheckOutputCapacity(int64_t ncodeunits) {
     if (ncodeunits > std::numeric_limits<offset_type>::max()) {
       return Status::CapacityError(
@@ -149,7 +125,7 @@ template <typename Type, typename StringTransform>
 struct StringTransformExec : public StringTransformExecBase<Type, StringTransform> {
   using StringTransformExecBase<Type, StringTransform>::Execute;
 
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     StringTransform transform;
     RETURN_NOT_OK(transform.PreExec(ctx, batch, out));
     return Execute(ctx, &transform, batch, out);
@@ -162,7 +138,7 @@ struct StringTransformExecWithState
   using State = typename StringTransform::State;
   using StringTransformExecBase<Type, StringTransform>::Execute;
 
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     StringTransform transform(State::Get(ctx));
     RETURN_NOT_OK(transform.PreExec(ctx, batch, out));
     return Execute(ctx, &transform, batch, out);
@@ -171,9 +147,9 @@ struct StringTransformExecWithState
 
 template <template <typename> class ExecFunctor>
 void MakeUnaryStringBatchKernel(
-    std::string name, FunctionRegistry* registry, const FunctionDoc* doc,
+    std::string name, FunctionRegistry* registry, FunctionDoc doc,
     MemAllocation::type mem_allocation = MemAllocation::PREALLOCATE) {
-  auto func = std::make_shared<ScalarFunction>(name, Arity::Unary(), doc);
+  auto func = std::make_shared<ScalarFunction>(name, Arity::Unary(), std::move(doc));
   for (const auto& ty : StringTypes()) {
     auto exec = GenerateVarBinaryToVarBinary<ExecFunctor>(ty);
     ScalarKernel kernel{{ty}, ty, std::move(exec)};
@@ -185,9 +161,9 @@ void MakeUnaryStringBatchKernel(
 
 template <template <typename> class ExecFunctor>
 void MakeUnaryStringBatchKernelWithState(
-    std::string name, FunctionRegistry* registry, const FunctionDoc* doc,
+    std::string name, FunctionRegistry* registry, FunctionDoc doc,
     MemAllocation::type mem_allocation = MemAllocation::PREALLOCATE) {
-  auto func = std::make_shared<ScalarFunction>(name, Arity::Unary(), doc);
+  auto func = std::make_shared<ScalarFunction>(name, Arity::Unary(), std::move(doc));
   {
     using t32 = ExecFunctor<StringType>;
     ScalarKernel kernel{{utf8()}, utf8(), t32::Exec, t32::State::Init};
@@ -213,8 +189,8 @@ static FunctionDoc StringPredicateDoc(std::string summary, std::string descripti
   return FunctionDoc{std::move(summary), std::move(description), {"strings"}};
 }
 
-static FunctionDoc StringClassifyDoc(std::string class_summary, std::string class_desc,
-                                     bool non_empty) {
+static inline FunctionDoc StringClassifyDoc(std::string class_summary,
+                                            std::string class_desc, bool non_empty) {
   std::string summary, description;
   {
     std::stringstream ss;
@@ -240,39 +216,26 @@ static FunctionDoc StringClassifyDoc(std::string class_summary, std::string clas
 
 template <typename Type, typename Predicate>
 struct StringPredicateFunctor {
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     Status st = Status::OK();
     EnsureUtf8LookupTablesFilled();
-    if (batch[0].kind() == Datum::ARRAY) {
-      const ArrayData& input = *batch[0].array();
-      ArrayIterator<Type> input_it(input);
-      ArrayData* out_arr = out->mutable_array();
-      ::arrow::internal::GenerateBitsUnrolled(
-          out_arr->buffers[1]->mutable_data(), out_arr->offset, input.length,
-          [&]() -> bool {
-            util::string_view val = input_it();
-            return Predicate::Call(ctx, reinterpret_cast<const uint8_t*>(val.data()),
-                                   val.size(), &st);
-          });
-    } else {
-      const auto& input = checked_cast<const BaseBinaryScalar&>(*batch[0].scalar());
-      if (input.is_valid) {
-        bool boolean_result = Predicate::Call(
-            ctx, input.value->data(), static_cast<size_t>(input.value->size()), &st);
-        // UTF decoding can lead to issues
-        if (st.ok()) {
-          out->value = std::make_shared<BooleanScalar>(boolean_result);
-        }
-      }
-    }
+    const ArraySpan& input = batch[0].array;
+    ArrayIterator<Type> input_it(input);
+    ArraySpan* out_arr = out->array_span();
+    ::arrow::internal::GenerateBitsUnrolled(
+        out_arr->buffers[1].data, out_arr->offset, input.length, [&]() -> bool {
+          std::string_view val = input_it();
+          return Predicate::Call(ctx, reinterpret_cast<const uint8_t*>(val.data()),
+                                 val.size(), &st);
+        });
     return st;
   }
 };
 
 template <typename Predicate>
 void AddUnaryStringPredicate(std::string name, FunctionRegistry* registry,
-                             const FunctionDoc* doc) {
-  auto func = std::make_shared<ScalarFunction>(name, Arity::Unary(), doc);
+                             FunctionDoc doc) {
+  auto func = std::make_shared<ScalarFunction>(name, Arity::Unary(), std::move(doc));
   for (const auto& ty : StringTypes()) {
     auto exec = GenerateVarBinaryToVarBinary<StringPredicateFunctor, Predicate>(ty);
     DCHECK_OK(func->AddKernel({ty}, boolean(), std::move(exec)));
@@ -288,7 +251,7 @@ struct StringSliceTransformBase : public StringTransformBase {
 
   const SliceOptions* options;
 
-  Status PreExec(KernelContext* ctx, const ExecBatch& batch, Datum* out) override {
+  Status PreExec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) override {
     options = &State::Get(ctx);
     if (options->step == 0) {
       return Status::Invalid("Slice step cannot be zero");
@@ -344,28 +307,20 @@ struct StringSplitExec {
   using State = OptionsWrapper<Options>;
 
   // Keep the temporary storage accross individual values, to minimize reallocations
-  std::vector<util::string_view> parts;
+  std::vector<std::string_view> parts;
   Options options;
 
   explicit StringSplitExec(const Options& options) : options(options) {}
 
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     return StringSplitExec{State::Get(ctx)}.Execute(ctx, batch, out);
   }
 
-  Status Execute(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
+  Status Execute(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     SplitFinder finder;
     RETURN_NOT_OK(finder.PreExec(options));
-    if (batch[0].kind() == Datum::ARRAY) {
-      return Execute(ctx, &finder, batch[0].array(), out);
-    }
-    DCHECK_EQ(batch[0].kind(), Datum::SCALAR);
-    return Execute(ctx, &finder, batch[0].scalar(), out);
-  }
-
-  Status Execute(KernelContext* ctx, SplitFinder* finder,
-                 const std::shared_ptr<ArrayData>& data, Datum* out) {
-    const ArrayType input(data);
+    // TODO(wesm): refactor to not require creating ArrayData
+    const ArrayType input(batch[0].array.ToArrayData());
 
     BuilderType builder(input.type(), ctx->memory_pool());
     // A slight overestimate of the data needed
@@ -373,7 +328,7 @@ struct StringSplitExec {
     // The minimum amount of strings needed
     RETURN_NOT_OK(builder.Resize(input.length() - input.null_count()));
 
-    ArrayData* output_list = out->mutable_array();
+    ArrayData* output_list = out->array_data().get();
     // List offsets were preallocated
     auto* list_offsets = output_list->GetMutableValues<list_offset_type>(1);
     DCHECK_NE(list_offsets, nullptr);
@@ -381,7 +336,7 @@ struct StringSplitExec {
     *list_offsets++ = 0;
     for (int64_t i = 0; i < input.length(); ++i) {
       if (!input.IsNull(i)) {
-        RETURN_NOT_OK(SplitString(input.GetView(i), finder, &builder));
+        RETURN_NOT_OK(SplitString(input.GetView(i), &finder, &builder));
         if (ARROW_PREDICT_FALSE(builder.length() >
                                 std::numeric_limits<list_offset_type>::max())) {
           return Status::CapacityError("List offset does not fit into 32 bit");
@@ -396,21 +351,7 @@ struct StringSplitExec {
     return Status::OK();
   }
 
-  Status Execute(KernelContext* ctx, SplitFinder* finder,
-                 const std::shared_ptr<Scalar>& scalar, Datum* out) {
-    const auto& input = checked_cast<const ScalarType&>(*scalar);
-    auto result = checked_cast<ListScalarType*>(out->scalar().get());
-    if (input.is_valid) {
-      result->is_valid = true;
-      BuilderType builder(input.type, ctx->memory_pool());
-      util::string_view s(*input.value);
-      RETURN_NOT_OK(SplitString(s, finder, &builder));
-      RETURN_NOT_OK(builder.Finish(&result->value));
-    }
-    return Status::OK();
-  }
-
-  Status SplitString(const util::string_view& s, SplitFinder* finder,
+  Status SplitString(const std::string_view& s, SplitFinder* finder,
                      BuilderType* builder) {
     const uint8_t* begin = reinterpret_cast<const uint8_t*>(s.data());
     const uint8_t* end = begin + s.length();
