@@ -19,6 +19,7 @@
 
 #include "arrow/engine/substrait/expression_internal.h"
 
+#include <memory>
 #include <utility>
 
 #include "arrow/builder.h"
@@ -28,7 +29,6 @@
 #include "arrow/engine/substrait/type_internal.h"
 #include "arrow/result.h"
 #include "arrow/status.h"
-#include "arrow/util/make_unique.h"
 #include "arrow/visit_scalar_inline.h"
 
 namespace arrow {
@@ -37,24 +37,123 @@ using internal::checked_cast;
 
 namespace engine {
 
-namespace internal {
-using ::arrow::internal::make_unique;
-}  // namespace internal
+namespace {
+
+Id NormalizeFunctionName(Id id) {
+  // Substrait plans encode the types into the function name so it might look like
+  // add:opt_i32_i32.  We don't care about  the :opt_i32_i32 so we just trim it
+  std::string_view func_name = id.name;
+  std::size_t colon_index = func_name.find_first_of(':');
+  if (colon_index != std::string_view::npos) {
+    func_name = func_name.substr(0, colon_index);
+  }
+  return {id.uri, func_name};
+}
+
+}  // namespace
+
+Status DecodeArg(const substrait::FunctionArgument& arg, uint32_t idx,
+                 SubstraitCall* call, const ExtensionSet& ext_set,
+                 const ConversionOptions& conversion_options) {
+  if (arg.has_enum_()) {
+    const substrait::FunctionArgument::Enum& enum_val = arg.enum_();
+    switch (enum_val.enum_kind_case()) {
+      case substrait::FunctionArgument::Enum::EnumKindCase::kSpecified:
+        call->SetEnumArg(idx, enum_val.specified());
+        break;
+      case substrait::FunctionArgument::Enum::EnumKindCase::kUnspecified:
+        call->SetEnumArg(idx, std::nullopt);
+        break;
+      default:
+        return Status::Invalid("Unrecognized enum kind case: ",
+                               enum_val.enum_kind_case());
+    }
+  } else if (arg.has_value()) {
+    ARROW_ASSIGN_OR_RAISE(compute::Expression expr,
+                          FromProto(arg.value(), ext_set, conversion_options));
+    call->SetValueArg(idx, std::move(expr));
+  } else if (arg.has_type()) {
+    return Status::NotImplemented("Type arguments not currently supported");
+  } else {
+    return Status::NotImplemented("Unrecognized function argument class");
+  }
+  return Status::OK();
+}
+
+Result<SubstraitCall> DecodeScalarFunction(
+    Id id, const substrait::Expression::ScalarFunction& scalar_fn,
+    const ExtensionSet& ext_set, const ConversionOptions& conversion_options) {
+  ARROW_ASSIGN_OR_RAISE(auto output_type_and_nullable,
+                        FromProto(scalar_fn.output_type(), ext_set, conversion_options));
+  SubstraitCall call(id, output_type_and_nullable.first, output_type_and_nullable.second);
+  for (int i = 0; i < scalar_fn.arguments_size(); i++) {
+    ARROW_RETURN_NOT_OK(DecodeArg(scalar_fn.arguments(i), static_cast<uint32_t>(i), &call,
+                                  ext_set, conversion_options));
+  }
+  return std::move(call);
+}
+
+std::string EnumToString(int value, const google::protobuf::EnumDescriptor* descriptor) {
+  const google::protobuf::EnumValueDescriptor* value_desc =
+      descriptor->FindValueByNumber(value);
+  if (value_desc == nullptr) {
+    return "unknown";
+  }
+  return value_desc->name();
+}
+
+Result<SubstraitCall> FromProto(const substrait::AggregateFunction& func, bool is_hash,
+                                const ExtensionSet& ext_set,
+                                const ConversionOptions& conversion_options) {
+  if (func.phase() != substrait::AggregationPhase::AGGREGATION_PHASE_INITIAL_TO_RESULT) {
+    return Status::NotImplemented(
+        "Unsupported aggregation phase '",
+        EnumToString(func.phase(), substrait::AggregationPhase_descriptor()),
+        "'.  Only INITIAL_TO_RESULT is supported");
+  }
+  if (func.invocation() !=
+      substrait::AggregateFunction::AggregationInvocation::
+          AggregateFunction_AggregationInvocation_AGGREGATION_INVOCATION_ALL) {
+    return Status::NotImplemented(
+        "Unsupported aggregation invocation '",
+        EnumToString(func.invocation(),
+                     substrait::AggregateFunction::AggregationInvocation_descriptor()),
+        "'.  Only AGGREGATION_INVOCATION_ALL is "
+        "supported");
+  }
+  if (func.sorts_size() > 0) {
+    return Status::NotImplemented("Aggregation sorts are not supported");
+  }
+  ARROW_ASSIGN_OR_RAISE(auto output_type_and_nullable,
+                        FromProto(func.output_type(), ext_set, conversion_options));
+  ARROW_ASSIGN_OR_RAISE(Id id, ext_set.DecodeFunction(func.function_reference()));
+  id = NormalizeFunctionName(id);
+  SubstraitCall call(id, output_type_and_nullable.first, output_type_and_nullable.second,
+                     is_hash);
+  for (int i = 0; i < func.arguments_size(); i++) {
+    ARROW_RETURN_NOT_OK(DecodeArg(func.arguments(i), static_cast<uint32_t>(i), &call,
+                                  ext_set, conversion_options));
+  }
+  return std::move(call);
+}
 
 Result<compute::Expression> FromProto(const substrait::Expression& expr,
-                                      const ExtensionSet& ext_set) {
+                                      const ExtensionSet& ext_set,
+                                      const ConversionOptions& conversion_options) {
   switch (expr.rex_type_case()) {
     case substrait::Expression::kLiteral: {
-      ARROW_ASSIGN_OR_RAISE(auto datum, FromProto(expr.literal(), ext_set));
+      ARROW_ASSIGN_OR_RAISE(auto datum,
+                            FromProto(expr.literal(), ext_set, conversion_options));
       return compute::literal(std::move(datum));
     }
 
     case substrait::Expression::kSelection: {
       if (!expr.selection().has_direct_reference()) break;
 
-      util::optional<compute::Expression> out;
+      std::optional<compute::Expression> out;
       if (expr.selection().has_expression()) {
-        ARROW_ASSIGN_OR_RAISE(out, FromProto(expr.selection().expression(), ext_set));
+        ARROW_ASSIGN_OR_RAISE(
+            out, FromProto(expr.selection().expression(), ext_set, conversion_options));
       }
 
       const auto* ref = &expr.selection().direct_reference();
@@ -126,9 +225,12 @@ Result<compute::Expression> FromProto(const substrait::Expression& expr,
       if (if_then.ifs_size() == 0) break;
 
       if (if_then.ifs_size() == 1) {
-        ARROW_ASSIGN_OR_RAISE(auto if_, FromProto(if_then.ifs(0).if_(), ext_set));
-        ARROW_ASSIGN_OR_RAISE(auto then, FromProto(if_then.ifs(0).then(), ext_set));
-        ARROW_ASSIGN_OR_RAISE(auto else_, FromProto(if_then.else_(), ext_set));
+        ARROW_ASSIGN_OR_RAISE(
+            auto if_, FromProto(if_then.ifs(0).if_(), ext_set, conversion_options));
+        ARROW_ASSIGN_OR_RAISE(
+            auto then, FromProto(if_then.ifs(0).then(), ext_set, conversion_options));
+        ARROW_ASSIGN_OR_RAISE(auto else_,
+                              FromProto(if_then.else_(), ext_set, conversion_options));
         return compute::call("if_else",
                              {std::move(if_), std::move(then), std::move(else_)});
       }
@@ -141,13 +243,16 @@ Result<compute::Expression> FromProto(const substrait::Expression& expr,
       args.reserve(if_then.ifs_size() + 2);
       args.emplace_back();
       for (const auto& if_ : if_then.ifs()) {
-        ARROW_ASSIGN_OR_RAISE(auto compute_if, FromProto(if_.if_(), ext_set));
-        ARROW_ASSIGN_OR_RAISE(auto compute_then, FromProto(if_.then(), ext_set));
+        ARROW_ASSIGN_OR_RAISE(auto compute_if,
+                              FromProto(if_.if_(), ext_set, conversion_options));
+        ARROW_ASSIGN_OR_RAISE(auto compute_then,
+                              FromProto(if_.then(), ext_set, conversion_options));
         conditions.emplace_back(std::move(compute_if));
         args.emplace_back(std::move(compute_then));
         condition_names.emplace_back("cond" + std::to_string(++name_counter));
       }
-      ARROW_ASSIGN_OR_RAISE(auto compute_else, FromProto(if_then.else_(), ext_set));
+      ARROW_ASSIGN_OR_RAISE(auto compute_else,
+                            FromProto(if_then.else_(), ext_set, conversion_options));
       args.emplace_back(std::move(compute_else));
       args[0] = compute::call("make_struct", std::move(conditions),
                               compute::MakeStructOptions(condition_names));
@@ -157,32 +262,25 @@ Result<compute::Expression> FromProto(const substrait::Expression& expr,
     case substrait::Expression::kScalarFunction: {
       const auto& scalar_fn = expr.scalar_function();
 
-      ARROW_ASSIGN_OR_RAISE(auto decoded_function,
+      ARROW_ASSIGN_OR_RAISE(Id function_id,
                             ext_set.DecodeFunction(scalar_fn.function_reference()));
+      function_id = NormalizeFunctionName(function_id);
+      ExtensionIdRegistry::SubstraitCallToArrow function_converter;
 
-      std::vector<compute::Expression> arguments(scalar_fn.arguments_size());
-      for (int i = 0; i < scalar_fn.arguments_size(); ++i) {
-        const auto& argument = scalar_fn.arguments(i);
-        switch (argument.arg_type_case()) {
-          case substrait::FunctionArgument::kValue: {
-            ARROW_ASSIGN_OR_RAISE(arguments[i], FromProto(argument.value(), ext_set));
-            break;
-          }
-          default:
-            return Status::NotImplemented(
-                "only value arguments are currently supported for functions");
-        }
-      }
-
-      auto func_name = decoded_function.name.to_string();
-      if (func_name != "cast") {
-        return compute::call(func_name, std::move(arguments));
+      if (function_id.uri.empty() || function_id.uri[0] == '/') {
+        // Currently the Substrait project has not aligned on a standard URI and often
+        // seems to use /.  In that case we fall back to name-only matching.
+        ARROW_ASSIGN_OR_RAISE(
+            function_converter,
+            ext_set.registry()->GetSubstraitCallToArrowFallback(function_id.name));
       } else {
-        ARROW_ASSIGN_OR_RAISE(auto output_type_desc,
-                              FromProto(scalar_fn.output_type(), ext_set));
-        auto cast_options = compute::CastOptions::Safe(std::move(output_type_desc.first));
-        return compute::call(func_name, std::move(arguments), std::move(cast_options));
+        ARROW_ASSIGN_OR_RAISE(function_converter,
+                              ext_set.registry()->GetSubstraitCallToArrow(function_id));
       }
+      ARROW_ASSIGN_OR_RAISE(
+          SubstraitCall substrait_call,
+          DecodeScalarFunction(function_id, scalar_fn, ext_set, conversion_options));
+      return function_converter(substrait_call);
     }
 
     default:
@@ -195,8 +293,10 @@ Result<compute::Expression> FromProto(const substrait::Expression& expr,
 }
 
 Result<Datum> FromProto(const substrait::Expression::Literal& lit,
-                        const ExtensionSet& ext_set) {
-  if (lit.nullable()) {
+                        const ExtensionSet& ext_set,
+                        const ConversionOptions& conversion_options) {
+  if (lit.nullable() &&
+      conversion_options.strictness == ConversionStrictness::EXACT_ROUNDTRIP) {
     // FIXME not sure how this field should be interpreted and there's no way to round
     // trip it through arrow
     return Status::Invalid(
@@ -295,7 +395,8 @@ Result<Datum> FromProto(const substrait::Expression::Literal& lit,
 
       ScalarVector fields(struct_.fields_size());
       for (int i = 0; i < struct_.fields_size(); ++i) {
-        ARROW_ASSIGN_OR_RAISE(auto field, FromProto(struct_.fields(i), ext_set));
+        ARROW_ASSIGN_OR_RAISE(auto field,
+                              FromProto(struct_.fields(i), ext_set, conversion_options));
         DCHECK(field.is_scalar());
         fields[i] = field.scalar();
       }
@@ -321,7 +422,8 @@ Result<Datum> FromProto(const substrait::Expression::Literal& lit,
 
       ScalarVector values(list.values_size());
       for (int i = 0; i < list.values_size(); ++i) {
-        ARROW_ASSIGN_OR_RAISE(auto value, FromProto(list.values(i), ext_set));
+        ARROW_ASSIGN_OR_RAISE(auto value,
+                              FromProto(list.values(i), ext_set, conversion_options));
         DCHECK(value.is_scalar());
         values[i] = value.scalar();
         if (element_type) {
@@ -360,8 +462,9 @@ Result<Datum> FromProto(const substrait::Expression::Literal& lit,
           return Status::Invalid("While converting to MapScalar encountered missing ",
                                  missing, " in ", map.DebugString());
         }
-        ARROW_ASSIGN_OR_RAISE(auto key, FromProto(kv.key(), ext_set));
-        ARROW_ASSIGN_OR_RAISE(auto value, FromProto(kv.value(), ext_set));
+        ARROW_ASSIGN_OR_RAISE(auto key, FromProto(kv.key(), ext_set, conversion_options));
+        ARROW_ASSIGN_OR_RAISE(auto value,
+                              FromProto(kv.value(), ext_set, conversion_options));
 
         DCHECK(key.is_scalar());
         DCHECK(value.is_scalar());
@@ -402,20 +505,22 @@ Result<Datum> FromProto(const substrait::Expression::Literal& lit,
     }
 
     case substrait::Expression::Literal::kEmptyList: {
-      ARROW_ASSIGN_OR_RAISE(auto type_nullable,
-                            FromProto(lit.empty_list().type(), ext_set));
+      ARROW_ASSIGN_OR_RAISE(auto type_nullable, FromProto(lit.empty_list().type(),
+                                                          ext_set, conversion_options));
       ARROW_ASSIGN_OR_RAISE(auto values, MakeEmptyArray(type_nullable.first));
       return ListScalar{std::move(values)};
     }
 
     case substrait::Expression::Literal::kEmptyMap: {
-      ARROW_ASSIGN_OR_RAISE(auto key_type_nullable,
-                            FromProto(lit.empty_map().key(), ext_set));
+      ARROW_ASSIGN_OR_RAISE(
+          auto key_type_nullable,
+          FromProto(lit.empty_map().key(), ext_set, conversion_options));
       ARROW_ASSIGN_OR_RAISE(auto keys,
                             MakeEmptyArray(std::move(key_type_nullable.first)));
 
-      ARROW_ASSIGN_OR_RAISE(auto value_type_nullable,
-                            FromProto(lit.empty_map().value(), ext_set));
+      ARROW_ASSIGN_OR_RAISE(
+          auto value_type_nullable,
+          FromProto(lit.empty_map().value(), ext_set, conversion_options));
       ARROW_ASSIGN_OR_RAISE(auto values,
                             MakeEmptyArray(std::move(value_type_nullable.first)));
 
@@ -430,7 +535,8 @@ Result<Datum> FromProto(const substrait::Expression::Literal& lit,
     }
 
     case substrait::Expression::Literal::kNull: {
-      ARROW_ASSIGN_OR_RAISE(auto type_nullable, FromProto(lit.null(), ext_set));
+      ARROW_ASSIGN_OR_RAISE(auto type_nullable,
+                            FromProto(lit.null(), ext_set, conversion_options));
       if (!type_nullable.second) {
         return Status::Invalid("Substrait null literal ", lit.DebugString(),
                                " is of non-nullable type");
@@ -526,7 +632,7 @@ struct ScalarToProtoImpl {
   Status Visit(const DayTimeIntervalScalar& s) { return NotImplemented(s); }
 
   Status Visit(const Decimal128Scalar& s) {
-    auto decimal = internal::make_unique<Lit::Decimal>();
+    auto decimal = std::make_unique<Lit::Decimal>();
 
     auto decimal_type = checked_cast<const Decimal128Type*>(s.type.get());
     decimal->set_precision(decimal_type->precision());
@@ -545,8 +651,8 @@ struct ScalarToProtoImpl {
 
   Status Visit(const ListScalar& s) {
     if (s.value->length() == 0) {
-      ARROW_ASSIGN_OR_RAISE(auto list_type,
-                            ToProto(*s.type, /*nullable=*/true, ext_set_));
+      ARROW_ASSIGN_OR_RAISE(auto list_type, ToProto(*s.type, /*nullable=*/true, ext_set_,
+                                                    conversion_options_));
       lit_->set_allocated_empty_list(list_type->release_list());
       return Status::OK();
     }
@@ -554,16 +660,17 @@ struct ScalarToProtoImpl {
     lit_->set_allocated_list(new Lit::List());
 
     const auto& list_type = checked_cast<const ListType&>(*s.type);
-    ARROW_ASSIGN_OR_RAISE(
-        auto element_type,
-        ToProto(*list_type.value_type(), list_type.value_field()->nullable(), ext_set_));
+    ARROW_ASSIGN_OR_RAISE(auto element_type, ToProto(*list_type.value_type(),
+                                                     list_type.value_field()->nullable(),
+                                                     ext_set_, conversion_options_));
 
     auto values = lit_->mutable_list()->mutable_values();
     values->Reserve(static_cast<int>(s.value->length()));
 
     for (int64_t i = 0; i < s.value->length(); ++i) {
       ARROW_ASSIGN_OR_RAISE(Datum list_element, s.value->GetScalar(i));
-      ARROW_ASSIGN_OR_RAISE(auto lit, ToProto(list_element, ext_set_));
+      ARROW_ASSIGN_OR_RAISE(auto lit,
+                            ToProto(list_element, ext_set_, conversion_options_));
       values->AddAllocated(lit.release());
     }
     return Status::OK();
@@ -576,7 +683,7 @@ struct ScalarToProtoImpl {
     fields->Reserve(static_cast<int>(s.value.size()));
 
     for (Datum field : s.value) {
-      ARROW_ASSIGN_OR_RAISE(auto lit, ToProto(field, ext_set_));
+      ARROW_ASSIGN_OR_RAISE(auto lit, ToProto(field, ext_set_, conversion_options_));
       fields->AddAllocated(lit.release());
     }
     return Status::OK();
@@ -588,7 +695,8 @@ struct ScalarToProtoImpl {
 
   Status Visit(const MapScalar& s) {
     if (s.value->length() == 0) {
-      ARROW_ASSIGN_OR_RAISE(auto map_type, ToProto(*s.type, /*nullable=*/true, ext_set_));
+      ARROW_ASSIGN_OR_RAISE(auto map_type, ToProto(*s.type, /*nullable=*/true, ext_set_,
+                                                   conversion_options_));
       lit_->set_allocated_empty_map(map_type->release_map());
       return Status::OK();
     }
@@ -601,14 +709,15 @@ struct ScalarToProtoImpl {
     key_values->Reserve(static_cast<int>(kv_arr.length()));
 
     for (int64_t i = 0; i < s.value->length(); ++i) {
-      auto kv = internal::make_unique<Lit::Map::KeyValue>();
+      auto kv = std::make_unique<Lit::Map::KeyValue>();
 
       ARROW_ASSIGN_OR_RAISE(Datum key_scalar, kv_arr.field(0)->GetScalar(i));
-      ARROW_ASSIGN_OR_RAISE(auto key, ToProto(key_scalar, ext_set_));
+      ARROW_ASSIGN_OR_RAISE(auto key, ToProto(key_scalar, ext_set_, conversion_options_));
       kv->set_allocated_key(key.release());
 
       ARROW_ASSIGN_OR_RAISE(Datum value_scalar, kv_arr.field(1)->GetScalar(i));
-      ARROW_ASSIGN_OR_RAISE(auto value, ToProto(value_scalar, ext_set_));
+      ARROW_ASSIGN_OR_RAISE(auto value,
+                            ToProto(value_scalar, ext_set_, conversion_options_));
       kv->set_allocated_value(value.release());
 
       key_values->AddAllocated(kv.release());
@@ -629,7 +738,7 @@ struct ScalarToProtoImpl {
     }
 
     if (auto length = UnwrapVarChar(*s.type)) {
-      auto var_char = internal::make_unique<Lit::VarChar>();
+      auto var_char = std::make_unique<Lit::VarChar>();
       var_char->set_length(*length);
       var_char->set_value(checked_cast<const StringScalar&>(*s.value).value->ToString());
 
@@ -644,7 +753,7 @@ struct ScalarToProtoImpl {
     };
 
     if (UnwrapIntervalYear(*s.type)) {
-      auto interval_year = internal::make_unique<Lit::IntervalYearToMonth>();
+      auto interval_year = std::make_unique<Lit::IntervalYearToMonth>();
       interval_year->set_years(GetPairOfInts().first);
       interval_year->set_months(GetPairOfInts().second);
 
@@ -653,7 +762,7 @@ struct ScalarToProtoImpl {
     }
 
     if (UnwrapIntervalDay(*s.type)) {
-      auto interval_day = internal::make_unique<Lit::IntervalDayToSecond>();
+      auto interval_day = std::make_unique<Lit::IntervalDayToSecond>();
       interval_day->set_days(GetPairOfInts().first);
       interval_day->set_seconds(GetPairOfInts().second);
 
@@ -680,22 +789,26 @@ struct ScalarToProtoImpl {
 
   substrait::Expression::Literal* lit_;
   ExtensionSet* ext_set_;
+  const ConversionOptions& conversion_options_;
 };
 }  // namespace
 
-Result<std::unique_ptr<substrait::Expression::Literal>> ToProto(const Datum& datum,
-                                                                ExtensionSet* ext_set) {
+Result<std::unique_ptr<substrait::Expression::Literal>> ToProto(
+    const Datum& datum, ExtensionSet* ext_set,
+    const ConversionOptions& conversion_options) {
   if (!datum.is_scalar()) {
     return Status::NotImplemented("representing ", datum.ToString(),
                                   " as a substrait::Expression::Literal");
   }
 
-  auto out = internal::make_unique<substrait::Expression::Literal>();
+  auto out = std::make_unique<substrait::Expression::Literal>();
 
   if (datum.scalar()->is_valid) {
-    RETURN_NOT_OK((ScalarToProtoImpl{out.get(), ext_set})(*datum.scalar()));
+    RETURN_NOT_OK(
+        (ScalarToProtoImpl{out.get(), ext_set, conversion_options})(*datum.scalar()));
   } else {
-    ARROW_ASSIGN_OR_RAISE(auto type, ToProto(*datum.type(), /*nullable=*/true, ext_set));
+    ARROW_ASSIGN_OR_RAISE(auto type, ToProto(*datum.type(), /*nullable=*/true, ext_set,
+                                             conversion_options));
     out->set_allocated_null(type.release());
   }
 
@@ -760,7 +873,7 @@ static Result<std::unique_ptr<substrait::Expression>> MakeDirectReference(
     }
   }
 
-  auto selection = internal::make_unique<substrait::Expression::FieldReference>();
+  auto selection = std::make_unique<substrait::Expression::FieldReference>();
   selection->set_allocated_direct_reference(ref_segment.release());
 
   if (expr && expr->rex_type_case() != substrait::Expression::REX_TYPE_NOT_SET) {
@@ -770,7 +883,7 @@ static Result<std::unique_ptr<substrait::Expression>> MakeDirectReference(
         new substrait::Expression::FieldReference::RootReference());
   }
 
-  auto out = internal::make_unique<substrait::Expression>();
+  auto out = std::make_unique<substrait::Expression>();
   out->set_allocated_selection(selection.release());
   return std::move(out);
 }
@@ -780,10 +893,10 @@ static Result<std::unique_ptr<substrait::Expression>> MakeDirectReference(
 static Result<std::unique_ptr<substrait::Expression>> MakeStructFieldReference(
     std::unique_ptr<substrait::Expression>&& expr, int field) {
   auto struct_field =
-      internal::make_unique<substrait::Expression::ReferenceSegment::StructField>();
+      std::make_unique<substrait::Expression::ReferenceSegment::StructField>();
   struct_field->set_field(field);
 
-  auto ref_segment = internal::make_unique<substrait::Expression::ReferenceSegment>();
+  auto ref_segment = std::make_unique<substrait::Expression::ReferenceSegment>();
   ref_segment->set_allocated_struct_field(struct_field.release());
 
   return MakeDirectReference(std::move(expr), std::move(ref_segment));
@@ -793,25 +906,61 @@ static Result<std::unique_ptr<substrait::Expression>> MakeStructFieldReference(
 static Result<std::unique_ptr<substrait::Expression>> MakeListElementReference(
     std::unique_ptr<substrait::Expression>&& expr, int offset) {
   auto list_element =
-      internal::make_unique<substrait::Expression::ReferenceSegment::ListElement>();
+      std::make_unique<substrait::Expression::ReferenceSegment::ListElement>();
   list_element->set_offset(offset);
 
-  auto ref_segment = internal::make_unique<substrait::Expression::ReferenceSegment>();
+  auto ref_segment = std::make_unique<substrait::Expression::ReferenceSegment>();
   ref_segment->set_allocated_list_element(list_element.release());
 
   return MakeDirectReference(std::move(expr), std::move(ref_segment));
 }
 
-Result<std::unique_ptr<substrait::Expression>> ToProto(const compute::Expression& expr,
-                                                       ExtensionSet* ext_set) {
+Result<std::unique_ptr<substrait::Expression::ScalarFunction>> EncodeSubstraitCall(
+    const SubstraitCall& call, ExtensionSet* ext_set,
+    const ConversionOptions& conversion_options) {
+  ARROW_ASSIGN_OR_RAISE(uint32_t anchor, ext_set->EncodeFunction(call.id()));
+  auto scalar_fn = std::make_unique<substrait::Expression::ScalarFunction>();
+  scalar_fn->set_function_reference(anchor);
+  ARROW_ASSIGN_OR_RAISE(
+      std::unique_ptr<substrait::Type> output_type,
+      ToProto(*call.output_type(), call.output_nullable(), ext_set, conversion_options));
+  scalar_fn->set_allocated_output_type(output_type.release());
+
+  for (uint32_t i = 0; i < call.size(); i++) {
+    substrait::FunctionArgument* arg = scalar_fn->add_arguments();
+    if (call.HasEnumArg(i)) {
+      auto enum_val = std::make_unique<substrait::FunctionArgument::Enum>();
+      ARROW_ASSIGN_OR_RAISE(std::optional<std::string_view> enum_arg, call.GetEnumArg(i));
+      if (enum_arg) {
+        enum_val->set_specified(std::string(*enum_arg));
+      } else {
+        enum_val->set_allocated_unspecified(new google::protobuf::Empty());
+      }
+      arg->set_allocated_enum_(enum_val.release());
+    } else if (call.HasValueArg(i)) {
+      ARROW_ASSIGN_OR_RAISE(compute::Expression value_arg, call.GetValueArg(i));
+      ARROW_ASSIGN_OR_RAISE(std::unique_ptr<substrait::Expression> value_expr,
+                            ToProto(value_arg, ext_set, conversion_options));
+      arg->set_allocated_value(value_expr.release());
+    } else {
+      return Status::Invalid("Call reported having ", call.size(),
+                             " arguments but no argument could be found at index ", i);
+    }
+  }
+  return std::move(scalar_fn);
+}
+
+Result<std::unique_ptr<substrait::Expression>> ToProto(
+    const compute::Expression& expr, ExtensionSet* ext_set,
+    const ConversionOptions& conversion_options) {
   if (!expr.IsBound()) {
     return Status::Invalid("ToProto requires a bound Expression");
   }
 
-  auto out = internal::make_unique<substrait::Expression>();
+  auto out = std::make_unique<substrait::Expression>();
 
   if (auto datum = expr.literal()) {
-    ARROW_ASSIGN_OR_RAISE(auto literal, ToProto(*datum, ext_set));
+    ARROW_ASSIGN_OR_RAISE(auto literal, ToProto(*datum, ext_set, conversion_options));
     out->set_allocated_literal(literal.release());
     return std::move(out);
   }
@@ -833,20 +982,21 @@ Result<std::unique_ptr<substrait::Expression>> ToProto(const compute::Expression
     auto conditions = call->arguments[0].call();
     if (conditions && conditions->function_name == "make_struct") {
       // catch the special case of calls convertible to IfThen
-      auto if_then_ = internal::make_unique<substrait::Expression::IfThen>();
+      auto if_then_ = std::make_unique<substrait::Expression::IfThen>();
 
       // don't try to convert argument 0 of the case_when; we have to convert the elements
       // of make_struct individually
       std::vector<std::unique_ptr<substrait::Expression>> arguments(
           call->arguments.size() - 1);
       for (size_t i = 1; i < call->arguments.size(); ++i) {
-        ARROW_ASSIGN_OR_RAISE(arguments[i - 1], ToProto(call->arguments[i], ext_set));
+        ARROW_ASSIGN_OR_RAISE(arguments[i - 1],
+                              ToProto(call->arguments[i], ext_set, conversion_options));
       }
 
       for (size_t i = 0; i < conditions->arguments.size(); ++i) {
-        ARROW_ASSIGN_OR_RAISE(auto cond_substrait,
-                              ToProto(conditions->arguments[i], ext_set));
-        auto clause = internal::make_unique<substrait::Expression::IfThen::IfClause>();
+        ARROW_ASSIGN_OR_RAISE(auto cond_substrait, ToProto(conditions->arguments[i],
+                                                           ext_set, conversion_options));
+        auto clause = std::make_unique<substrait::Expression::IfThen::IfClause>();
         clause->set_allocated_if_(cond_substrait.release());
         clause->set_allocated_then(arguments[i].release());
         if_then_->mutable_ifs()->AddAllocated(clause.release());
@@ -863,7 +1013,8 @@ Result<std::unique_ptr<substrait::Expression>> ToProto(const compute::Expression
   // should be able to convert all its arguments first here
   std::vector<std::unique_ptr<substrait::Expression>> arguments(call->arguments.size());
   for (size_t i = 0; i < arguments.size(); ++i) {
-    ARROW_ASSIGN_OR_RAISE(arguments[i], ToProto(call->arguments[i], ext_set));
+    ARROW_ASSIGN_OR_RAISE(arguments[i],
+                          ToProto(call->arguments[i], ext_set, conversion_options));
   }
 
   if (call->function_name == "struct_field") {
@@ -883,7 +1034,7 @@ Result<std::unique_ptr<substrait::Expression>> ToProto(const compute::Expression
     if (arguments[0]->has_selection() &&
         arguments[0]->selection().has_direct_reference()) {
       if (arguments[1]->has_literal() && arguments[1]->literal().literal_type_case() ==
-                                             substrait::Expression_Literal::kI32) {
+                                             substrait::Expression::Literal::kI32) {
         return MakeListElementReference(std::move(arguments[0]),
                                         arguments[1]->literal().i32());
       }
@@ -892,11 +1043,11 @@ Result<std::unique_ptr<substrait::Expression>> ToProto(const compute::Expression
 
   if (call->function_name == "if_else") {
     // catch the special case of calls convertible to IfThen
-    auto if_clause = internal::make_unique<substrait::Expression::IfThen::IfClause>();
+    auto if_clause = std::make_unique<substrait::Expression::IfThen::IfClause>();
     if_clause->set_allocated_if_(arguments[0].release());
     if_clause->set_allocated_then(arguments[1].release());
 
-    auto if_then = internal::make_unique<substrait::Expression::IfThen>();
+    auto if_then = std::make_unique<substrait::Expression::IfThen>();
     if_then->mutable_ifs()->AddAllocated(if_clause.release());
     if_then->set_allocated_else_(arguments[2].release());
 
@@ -905,17 +1056,12 @@ Result<std::unique_ptr<substrait::Expression>> ToProto(const compute::Expression
   }
 
   // other expression types dive into extensions immediately
-  ARROW_ASSIGN_OR_RAISE(auto anchor, ext_set->EncodeFunction(call->function_name));
-
-  auto scalar_fn = internal::make_unique<substrait::Expression::ScalarFunction>();
-  scalar_fn->set_function_reference(anchor);
-  scalar_fn->mutable_arguments()->Reserve(static_cast<int>(arguments.size()));
-  for (auto& arg : arguments) {
-    auto argument = internal::make_unique<substrait::FunctionArgument>();
-    argument->set_allocated_value(arg.release());
-    scalar_fn->mutable_arguments()->AddAllocated(argument.release());
-  }
-
+  ARROW_ASSIGN_OR_RAISE(
+      ExtensionIdRegistry::ArrowToSubstraitCall converter,
+      ext_set->registry()->GetArrowToSubstraitCall(call->function_name));
+  ARROW_ASSIGN_OR_RAISE(SubstraitCall substrait_call, converter(*call));
+  ARROW_ASSIGN_OR_RAISE(std::unique_ptr<substrait::Expression::ScalarFunction> scalar_fn,
+                        EncodeSubstraitCall(substrait_call, ext_set, conversion_options));
   out->set_allocated_scalar_function(scalar_fn.release());
   return std::move(out);
 }
