@@ -17,6 +17,7 @@
 
 #include "arrow/engine/substrait/type_internal.h"
 
+#include <memory>
 #include <string>
 #include <vector>
 
@@ -25,16 +26,11 @@
 #include "arrow/status.h"
 #include "arrow/type.h"
 #include "arrow/util/logging.h"
-#include "arrow/util/make_unique.h"
 #include "arrow/visit_type_inline.h"
 
 namespace arrow {
 namespace engine {
 
-namespace internal {
-using ::arrow::internal::make_unique;
-}  // namespace internal
-
 namespace {
 
 template <typename TypeMessage>
@@ -62,7 +58,8 @@ Result<std::pair<std::shared_ptr<DataType>, bool>> FromProtoImpl(
 template <typename Types, typename NextName>
 Result<FieldVector> FieldsFromProto(int size, const Types& types,
                                     const NextName& next_name,
-                                    const ExtensionSet& ext_set) {
+                                    const ExtensionSet& ext_set,
+                                    const ConversionOptions& conversion_options) {
   FieldVector fields(size);
   for (int i = 0; i < size; ++i) {
     std::string name = next_name();
@@ -72,13 +69,14 @@ Result<FieldVector> FieldsFromProto(int size, const Types& types,
     if (types.Get(i).has_struct_()) {
       const auto& struct_ = types.Get(i).struct_();
 
-      ARROW_ASSIGN_OR_RAISE(
-          type, FieldsFromProto(struct_.types_size(), struct_.types(), next_name, ext_set)
-                    .Map(arrow::struct_));
+      ARROW_ASSIGN_OR_RAISE(type, FieldsFromProto(struct_.types_size(), struct_.types(),
+                                                  next_name, ext_set, conversion_options)
+                                      .Map(arrow::struct_));
 
       nullable = IsNullable(struct_);
     } else {
-      ARROW_ASSIGN_OR_RAISE(std::tie(type, nullable), FromProto(types.Get(i), ext_set));
+      ARROW_ASSIGN_OR_RAISE(std::tie(type, nullable),
+                            FromProto(types.Get(i), ext_set, conversion_options));
     }
 
     fields[i] = field(std::move(name), std::move(type), nullable);
@@ -89,7 +87,8 @@ Result<FieldVector> FieldsFromProto(int size, const Types& types,
 }  // namespace
 
 Result<std::pair<std::shared_ptr<DataType>, bool>> FromProto(
-    const ::substrait::Type& type, const ExtensionSet& ext_set) {
+    const ::substrait::Type& type, const ExtensionSet& ext_set,
+    const ConversionOptions& conversion_options) {
   switch (type.kind_case()) {
     case ::substrait::Type::kBool:
       return FromProtoImpl<BooleanType>(type.bool_());
@@ -151,9 +150,10 @@ Result<std::pair<std::shared_ptr<DataType>, bool>> FromProto(
     case ::substrait::Type::kStruct: {
       const auto& struct_ = type.struct_();
 
-      ARROW_ASSIGN_OR_RAISE(auto fields, FieldsFromProto(
-                                             struct_.types_size(), struct_.types(),
-                                             /*next_name=*/[] { return ""; }, ext_set));
+      ARROW_ASSIGN_OR_RAISE(
+          auto fields, FieldsFromProto(
+                           struct_.types_size(), struct_.types(),
+                           /*next_name=*/[] { return ""; }, ext_set, conversion_options));
 
       return FromProtoImpl<StructType>(struct_, std::move(fields));
     }
@@ -167,7 +167,8 @@ Result<std::pair<std::shared_ptr<DataType>, bool>> FromProto(
             list.DebugString());
       }
 
-      ARROW_ASSIGN_OR_RAISE(auto type_nullable, FromProto(list.type(), ext_set));
+      ARROW_ASSIGN_OR_RAISE(auto type_nullable,
+                            FromProto(list.type(), ext_set, conversion_options));
       return FromProtoImpl<ListType>(
           list, field("item", std::move(type_nullable.first), type_nullable.second));
     }
@@ -182,8 +183,10 @@ Result<std::pair<std::shared_ptr<DataType>, bool>> FromProto(
                                missing, " type in ", map.DebugString());
       }
 
-      ARROW_ASSIGN_OR_RAISE(auto key_nullable, FromProto(map.key(), ext_set));
-      ARROW_ASSIGN_OR_RAISE(auto value_nullable, FromProto(map.value(), ext_set));
+      ARROW_ASSIGN_OR_RAISE(auto key_nullable,
+                            FromProto(map.key(), ext_set, conversion_options));
+      ARROW_ASSIGN_OR_RAISE(auto value_nullable,
+                            FromProto(map.value(), ext_set, conversion_options));
 
       if (key_nullable.second) {
         return Status::Invalid(
@@ -296,8 +299,8 @@ struct DataTypeToProtoImpl {
 
   Status Visit(const ListType& t) {
     // FIXME assert default field name; custom ones won't roundtrip
-    ARROW_ASSIGN_OR_RAISE(
-        auto type, ToProto(*t.value_type(), t.value_field()->nullable(), ext_set_));
+    ARROW_ASSIGN_OR_RAISE(auto type, ToProto(*t.value_type(), t.value_field()->nullable(),
+                                             ext_set_, conversion_options_));
     SetWithThen(&::substrait::Type::set_allocated_list)
         ->set_allocated_type(type.release());
     return Status::OK();
@@ -313,8 +316,8 @@ struct DataTypeToProtoImpl {
         return Status::Invalid(
             "::substrait::Type::Struct does not support field metadata");
       }
-      ARROW_ASSIGN_OR_RAISE(auto type,
-                            ToProto(*field->type(), field->nullable(), ext_set_));
+      ARROW_ASSIGN_OR_RAISE(auto type, ToProto(*field->type(), field->nullable(),
+                                               ext_set_, conversion_options_));
       types->AddAllocated(type.release());
     }
     return Status::OK();
@@ -328,11 +331,12 @@ struct DataTypeToProtoImpl {
     // FIXME assert default field names; custom ones won't roundtrip
     auto map = SetWithThen(&::substrait::Type::set_allocated_map);
 
-    ARROW_ASSIGN_OR_RAISE(auto key, ToProto(*t.key_type(), /*nullable=*/false, ext_set_));
+    ARROW_ASSIGN_OR_RAISE(auto key, ToProto(*t.key_type(), /*nullable=*/false, ext_set_,
+                                            conversion_options_));
     map->set_allocated_key(key.release());
 
-    ARROW_ASSIGN_OR_RAISE(auto value,
-                          ToProto(*t.item_type(), t.item_field()->nullable(), ext_set_));
+    ARROW_ASSIGN_OR_RAISE(auto value, ToProto(*t.item_type(), t.item_field()->nullable(),
+                                              ext_set_, conversion_options_));
     map->set_allocated_value(value.release());
 
     return Status::OK();
@@ -373,7 +377,7 @@ struct DataTypeToProtoImpl {
 
   template <typename Sub>
   Sub* SetWithThen(void (::substrait::Type::*set_allocated_sub)(Sub*)) {
-    auto sub = internal::make_unique<Sub>();
+    auto sub = std::make_unique<Sub>();
     sub->set_nullability(nullable_ ? ::substrait::Type::NULLABILITY_NULLABLE
                                    : ::substrait::Type::NULLABILITY_REQUIRED);
 
@@ -390,7 +394,7 @@ struct DataTypeToProtoImpl {
   template <typename T>
   Status EncodeUserDefined(const T& t) {
     ARROW_ASSIGN_OR_RAISE(auto anchor, ext_set_->EncodeType(t));
-    auto user_defined = internal::make_unique<::substrait::Type_UserDefined>();
+    auto user_defined = std::make_unique<::substrait::Type::UserDefined>();
     user_defined->set_type_reference(anchor);
     user_defined->set_nullability(nullable_ ? ::substrait::Type::NULLABILITY_NULLABLE
                                             : ::substrait::Type::NULLABILITY_REQUIRED);
@@ -407,18 +411,22 @@ struct DataTypeToProtoImpl {
   ::substrait::Type* type_;
   bool nullable_;
   ExtensionSet* ext_set_;
+  const ConversionOptions& conversion_options_;
 };
 }  // namespace
 
-Result<std::unique_ptr<::substrait::Type>> ToProto(const DataType& type, bool nullable,
-                                                   ExtensionSet* ext_set) {
-  auto out = internal::make_unique<::substrait::Type>();
-  RETURN_NOT_OK((DataTypeToProtoImpl{out.get(), nullable, ext_set})(type));
+Result<std::unique_ptr<::substrait::Type>> ToProto(
+    const DataType& type, bool nullable, ExtensionSet* ext_set,
+    const ConversionOptions& conversion_options) {
+  auto out = std::make_unique<::substrait::Type>();
+  RETURN_NOT_OK(
+      (DataTypeToProtoImpl{out.get(), nullable, ext_set, conversion_options})(type));
   return std::move(out);
 }
 
 Result<std::shared_ptr<Schema>> FromProto(const ::substrait::NamedStruct& named_struct,
-                                          const ExtensionSet& ext_set) {
+                                          const ExtensionSet& ext_set,
+                                          const ConversionOptions& conversion_options) {
   if (!named_struct.has_struct_()) {
     return Status::Invalid("While converting ", named_struct.DebugString(),
                            " no anonymous struct type was provided to which names "
@@ -436,7 +444,7 @@ Result<std::shared_ptr<Schema>> FromProto(const ::substrait::NamedStruct& named_
                                                       ? named_struct.names().Get(i)
                                                       : "";
                                          },
-                                         ext_set));
+                                         ext_set, conversion_options));
 
   if (requested_names_count != named_struct.names_size()) {
     return Status::Invalid("While converting ", named_struct.DebugString(), " received ",
@@ -460,19 +468,20 @@ void ToProtoGetDepthFirstNames(const FieldVector& fields,
 }
 }  // namespace
 
-Result<std::unique_ptr<::substrait::NamedStruct>> ToProto(const Schema& schema,
-                                                          ExtensionSet* ext_set) {
+Result<std::unique_ptr<::substrait::NamedStruct>> ToProto(
+    const Schema& schema, ExtensionSet* ext_set,
+    const ConversionOptions& conversion_options) {
   if (schema.metadata()) {
     return Status::Invalid("::substrait::NamedStruct does not support schema metadata");
   }
 
-  auto named_struct = internal::make_unique<::substrait::NamedStruct>();
+  auto named_struct = std::make_unique<::substrait::NamedStruct>();
 
   auto names = named_struct->mutable_names();
   names->Reserve(schema.num_fields());
   ToProtoGetDepthFirstNames(schema.fields(), names);
 
-  auto struct_ = internal::make_unique<::substrait::Type::Struct>();
+  auto struct_ = std::make_unique<::substrait::Type::Struct>();
   auto types = struct_->mutable_types();
   types->Reserve(schema.num_fields());
 
@@ -481,7 +490,8 @@ Result<std::unique_ptr<::substrait::NamedStruct>> ToProto(const Schema& schema,
       return Status::Invalid("::substrait::NamedStruct does not support field metadata");
     }
 
-    ARROW_ASSIGN_OR_RAISE(auto type, ToProto(*field->type(), field->nullable(), ext_set));
+    ARROW_ASSIGN_OR_RAISE(auto type, ToProto(*field->type(), field->nullable(), ext_set,
+                                             conversion_options));
     types->AddAllocated(type.release());
   }
 
